package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.ConexionDAO;
import com.rjconsultores.ventaboletos.entidad.Conexion;
import com.rjconsultores.ventaboletos.entidad.ConexionCtrlTemp;
import com.rjconsultores.ventaboletos.entidad.ConexionTemp;

@Repository("conexionDAO")
public class ConexionHibernateDAO extends GenericHibernateDAO<Conexion, Integer> implements ConexionDAO {
	private static Logger log = Logger.getLogger(ConexionHibernateDAO.class);

	 @Autowired
	    public ConexionHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
	        setSessionFactory(factory);
	    }

	
	@Override
	public void generarConexiones(Integer usuarioId) {
		this.getSession().createQuery("DELETE ConexionTemp");
		this.getSession().createQuery("DELETE ConexionCtrlTemp");

		Query queryParadas = this.getSession().createQuery("select t.origem.paradaId,t.destino.paradaId from Tramo t where t.activo = 1 order by t.origem.paradaId ");
		List<Object[]> lsParadas = queryParadas.list();

		StringBuilder sb = new StringBuilder("");
		sb.append("SELECT ");
		sb.append("	t.origem.paradaId, ");
		sb.append("	t.destino.paradaId, ");
		sb.append("	t2.origem.paradaId, ");
		sb.append("	t2.destino.paradaId, ");
		sb.append("    MAX(t.kmReal), ");
		sb.append("    MAX(tcs.tiemporecorrido), ");
		sb.append("    MAX(t2.kmReal), ");
		sb.append("    MAX(tcs2.tiemporecorrido) ");
		sb.append(" ");
		sb.append("FROM ");
		sb.append("    Tramo t, ");
		sb.append("    Tramo t2, ");
		sb.append("    TramoServicio tcs, ");
		sb.append("    TramoServicio tcs2 ");
		sb.append("     ");
		sb.append("where ");
		sb.append("	t.origem.paradaId = :origen ");
		sb.append("	and t.activo=1 ");
		sb.append("	 ");
		sb.append("	and t2.destino.paradaId = :destino ");
		sb.append("	and t2.activo=1 ");
		sb.append("	and t2.origem= t.destino ");
		sb.append("	 ");
		sb.append("	and tcs.tramo=t ");
		sb.append("	and tcs.activo=1 ");
		sb.append("  ");
		sb.append("	and tcs2.tramo=t2 ");
		sb.append("	and tcs2.activo=1 ");
		sb.append("  ");
		sb.append("group by ");
		sb.append("    t.origem.paradaId, ");
		sb.append("    t.destino.paradaId, ");
		sb.append("    t2.origem.paradaId, ");
		sb.append("    t2.destino.paradaId ");

		Map<String, Long> mapConexionCtrlId = new LinkedHashMap<String, Long>();
		int grupo = 0;
		for (Object[] obj : lsParadas) {
			Integer origenId = (Integer) obj[0];
			Integer destinoId = (Integer) obj[1];

			Query query = getSession().createQuery(sb.toString());
			query.setParameter("origen", origenId);
			query.setParameter("destino", destinoId);

			List<Object[]> list = query.list();

			for (Object[] objConexion : list) {
				Integer origen1 = (Integer) objConexion[0];
				Integer destino1 = (Integer) objConexion[1];
				Integer origen2 = (Integer) objConexion[2];
				Integer destino2 = (Integer) objConexion[3];

				String claveConexionCtrl = origen1 + "-" + destino2;

				Long conexionCtrlId = mapConexionCtrlId.get(claveConexionCtrl);

				if (conexionCtrlId == null) {
					ConexionCtrlTemp c = new ConexionCtrlTemp();
					c.setActivo(true);
					c.setOrigenId(origen1);
					c.setDestinoId(destino2);
					c.setFecmodif(new java.util.Date());
					c.setUsuarioId(usuarioId);

					conexionCtrlId=(Long) this.getSession().save(c);
					mapConexionCtrlId.put(claveConexionCtrl, conexionCtrlId);

				}
				grupo++;

				ConexionTemp cTemp = new ConexionTemp();
				cTemp.setActivo(true);
				cTemp.setConexionctrlId(conexionCtrlId);
				cTemp.setOrigenId(origen1);
				cTemp.setDestinoId(destino1);
				cTemp.setNumgrupo(grupo);
				cTemp.setNumsecuencia((short) 1);
				cTemp.setUsuarioId(usuarioId);

				ConexionTemp cTemp2 = new ConexionTemp();
				cTemp2.setActivo(true);
				cTemp2.setConexionctrlId(conexionCtrlId);
				cTemp2.setOrigenId(origen2);
				cTemp2.setDestinoId(destino2);
				cTemp2.setNumgrupo(grupo);
				cTemp2.setNumsecuencia((short) 2);
				cTemp2.setUsuarioId(usuarioId);

				getSession().save(cTemp);
				getSession().save(cTemp2);
			}

		}
	}
}
